@@ -1,8 +1,9 @@
-/* $Id: sim4.init.c,v 1.78 2007/04/24 08:52:53 c4chris Exp $
+/* $Id: sim4.init.c,v 1.85 2008/04/16 14:32:48 c4chris Exp $
  *
  * Christian Iseli, LICR ITO, Christian.Iseli@licr.org
  *
- * Copyright (c) 2001-2006 Swiss Institute of Bioinformatics.
+ * Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
+ * Swiss Institute of Bioinformatics.
  * Copyright (C) 1998-2001  Liliana Florea.
  */
 
@@ -67,7 +68,7 @@ static void free_seq(seq_p_t);
 
 static const char Usage[] =
 "%s [options] dna est_db\n\n"
-"This is SIBsim4 version 0.15.\n\n"
+"This is SIBsim4 version 0.17.\n\n"
 #ifdef DEBUG
 "Debug version\n\n"
 #endif
@@ -77,6 +78,8 @@ static const char Usage[] =
 "             1: alignment text\n"
 "             3: both exon endpoints and alignment text\n"
 "             4: both exon endpoints and alignment text with polyA info\n"
+"             5: same as 4 plus display chimeric alignment text\n"
+"                (which is currently problematic)\n"
 "            Note that 2 is unimplemented [%d]\n"
 "  -C <int>  MSP score threshold for the second pass [%d]\n"
 "  -c <int>  minimum score cutoff [%d]\n"
@@ -84,6 +87,10 @@ static const char Usage[] =
 "  -f <int>  score filter in percent (0 to disable filtering) [%d]\n"
 "  -g <int>  join exons when gap on genomic and RNA have lengths which\n"
 "            differ at most by this percentage [%d]\n"
+"  -H <int>  report chimeric transcripts when the best score is lower\n"
+"            than this percentage of the overall RNA coverage and the\n"
+"            chimera score is greater than this percentage of the\n"
+"            RNA length (0 disables this report) [%u]\n"
 "  -I <int>  window width in which to search for intron splicing [%d]\n"
 "  -K <int>  MSP score threshold for the first pass [%d]\n"
 "  -L <str>  a comma separated list of forward splice-types [%s]\n"
@@ -127,7 +134,7 @@ main(int argc, char *argv[])
   options.gapPct = DEFAULT_GAPPCT;
   options.intron_window = 6;
   options.K = DEFAULT_K;
-  options.splice_type_list = "GTAG,GCAG,GTAC,ATAC";
+  options.splice_type_list = (unsigned char *) "GTAG,GCAG,GTAC,ATAC";
   options.nbSplice = 4;
   options.scoreSplice_window = 10;
   options.mismatchScore = MISMATCH;
@@ -138,28 +145,29 @@ main(int argc, char *argv[])
   options.filterPct = DEFAULT_FILTER;
   options.minScore_cutoff = MATCH_CUTOFF;
   options.splitScorePct = DEFAULT_SPLIT_SCORE;
+  options.huntChimera = DEFAULT_CHIMERA;
   while (1) {
-    int c = getopt(argc, argv, "A:C:c:E:f:g:I:K:L:M:o:q:R:r:s:W:X:");
+    int c = getopt(argc, argv, "A:C:c:E:f:g:H:I:K:L:M:o:q:R:r:s:W:X:");
     if (c == -1)
       break;
     switch (c) {
     case 'A':
       options.ali_flag = atoi(optarg);
-      if (options.ali_flag < 0 || options.ali_flag > 4)
-	fatal("A must be one of 0, 1, 2, 3, or 4.\n");
+      if (options.ali_flag < 0 || options.ali_flag > 5)
+	fatal("A must be one of 0, 1, 2, 3, 4, or 5.\n");
       break;
     case 'C': {
       int val = atoi(optarg);
       if (val < 0)
 	fatal("Value for option C must be non-negative.\n");
-      options.C = val;
+      options.C = (unsigned int) val;
       break;
     }
     case 'c': {
       int val = atoi(optarg);
       if (val < 0)
 	fatal("Value for option c must be non-negative.\n");
-      options.minScore_cutoff = val;
+      options.minScore_cutoff = (unsigned int) val;
       break;
     }
     case 'E':
@@ -168,27 +176,38 @@ main(int argc, char *argv[])
 	fatal("Cutoff (E) must be within [3,10].\n");
       break;
     case 'f':
-      options.filterPct = atoi(optarg);
+      options.filterPct = (unsigned int) atoi(optarg);
       if (options.filterPct > 100)
 	fatal("Filter in percent (f) must be within [0,100].\n");
       break;
     case 'g':
-      options.gapPct = atoi(optarg);
+      options.gapPct = (unsigned int) atoi(optarg);
+      if (options.gapPct > 100)
+	fatal("Length difference in percent (g) must be within [0,100].\n");
       break;
-    case 'I':
-      options.intron_window = atoi(optarg);
+    case 'H':
+      options.huntChimera = (unsigned int) atoi(optarg);
+      if (options.huntChimera > 100)
+	fatal("Chimera filter in percent (H) must be within [0,100].\n");
       break;
+    case 'I': {
+      int val = atoi(optarg);
+      if (val < 0)
+	fatal("Value for option I must be non-negative.\n");
+      options.intron_window = (unsigned int) val;
+      break;
+    }
     case 'K': {
       int val = atoi(optarg);
       if (val < 0)
 	fatal("Value for option K must be non-negative.\n");
-      options.K = val;
+      options.K = (unsigned int) val;
       break;
     }
     case 'L': {
       size_t i;
       size_t len = strlen(optarg);
-      options.splice_type_list = optarg;
+      options.splice_type_list = (unsigned char *) optarg;
       options.nbSplice = 1;
       if (len % 5 != 4)
 	fatal("Splice types list has illegal length (%zu)\n", len);
@@ -214,35 +233,39 @@ main(int argc, char *argv[])
       int val = atoi(optarg);
       if (val < 0)
 	fatal("Value for option M must be non-negative.\n");
-      options.scoreSplice_window = val;
+      options.scoreSplice_window = (unsigned int) val;
       break;
     }
-    case 'o':
-      options.dnaOffset = atoi(optarg);
+    case 'o': {
+      int val = atoi(optarg);
+      if (val < 0)
+	fatal("Value for option o must be non-negative.\n");
+      options.dnaOffset = (unsigned int) val;
       break;
+    }
     case 'q':
       options.mismatchScore = atoi(optarg);
       break;
     case 'R':
-      options.reverse = atoi(optarg);
-      if (options.reverse < 0 || options.reverse > 2)
+      options.reverse = (unsigned int) atoi(optarg);
+      if (options.reverse > 2)
 	fatal("R must be one of 0, 1, or 2.\n");
       break;
     case 'r':
       options.matchScore = atoi(optarg);
       break;
     case 's':
-      options.splitScorePct = atoi(optarg);
+      options.splitScorePct = (unsigned int) atoi(optarg);
       if (options.splitScorePct > 100)
 	fatal("Split score in percent (s) must be within [0,100].\n");
       break;
     case 'W':
-      options.W = atoi(optarg);
+      options.W = (unsigned int) atoi(optarg);
       if (options.W < 1 || options.W > 15)
 	fatal("W must be within [1,15].\n");
       break;
     case 'X':
-      options.X = atoi(optarg);
+      options.X = (unsigned int) atoi(optarg);
       if (options.X < 1)
 	fatal("X must be positive.\n");
       break;
@@ -253,12 +276,25 @@ main(int argc, char *argv[])
     }
   }
   if (optind + 2 != argc) {
-    fprintf(stderr, Usage, argv[0], options.ali_flag, options.C,
-	    options.minScore_cutoff, options.cutoff,
-	    options.filterPct, options.gapPct, options.intron_window,
-	    options.K, options.splice_type_list, options.scoreSplice_window,
-	    options.dnaOffset, options.mismatchScore, options.reverse,
-	    options.matchScore, options.splitScorePct, options.W, options.X);
+    fprintf(stderr, Usage, argv[0],
+	    options.ali_flag,
+	    options.C,
+	    options.minScore_cutoff,
+	    options.cutoff,
+	    options.filterPct,
+	    options.gapPct,
+	    options.huntChimera,
+	    options.intron_window,
+	    options.K,
+	    options.splice_type_list,
+	    options.scoreSplice_window,
+	    options.dnaOffset,
+	    options.mismatchScore,
+	    options.reverse,
+	    options.matchScore,
+	    options.splitScorePct,
+	    options.W,
+	    options.X);
     return 1;
   }
 
@@ -266,7 +302,7 @@ main(int argc, char *argv[])
   init_seq(argv[optind], &seq1);
   if (get_next_seq(&seq1, options.dnaOffset, 1) != 0)
     fatal("Cannot read sequence from %s.\n", argv[optind]);
-  strncpy(dna_seq_head, seq1.header, 256);
+  strncpy(dna_seq_head, seq1.header, 256UL);
 
   /* read seq2 */
   init_seq(argv[optind + 1], &seq2);
@@ -283,7 +319,7 @@ main(int argc, char *argv[])
   count = 0;
   while (!count || get_next_seq(&seq2, 0, 0) == 0) {
     unsigned int curRes;
-    strncpy(rna_seq_head, seq2.header, 256);
+    strncpy(rna_seq_head, seq2.header, 256UL);
     ++count;
 
     switch (options.reverse) {
@@ -299,6 +335,57 @@ main(int argc, char *argv[])
     default:
       fatal ("Unrecognized request for EST orientation.\n");
     }
+    /* Show chimeric result if requested.  */
+    if (options.huntChimera > 0) {
+      unsigned int max_nmatches = 0;
+      unsigned int max_cmatches = 0;
+      for (curRes = 0; curRes < rev_res.nb; curRes++) {
+	result_p_t r = rev_res.e.result[curRes];
+	if (r->chimera) {
+	  if (r->st.nmatches > max_cmatches)
+	    max_cmatches = r->st.nmatches;
+	} else {
+	  if (r->st.nmatches > max_nmatches)
+	    max_nmatches = r->st.nmatches;
+	}
+      }
+      for (curRes = 0; curRes < res.nb; curRes++) {
+	result_p_t r = res.e.result[curRes];
+	if (r->chimera) {
+	  if (r->st.nmatches > max_cmatches)
+	    max_cmatches = r->st.nmatches;
+	} else {
+	  if (r->st.nmatches > max_nmatches)
+	    max_nmatches = r->st.nmatches;
+	}
+      }
+      if ((max_cmatches * options.huntChimera) / 100 > max_nmatches
+	  && max_cmatches > (seq2.len * options.huntChimera) / 100) {
+	/* The chimera wins...  */
+	for (curRes = 0; curRes < rev_res.nb; curRes++) {
+	  result_p_t r = rev_res.e.result[curRes];
+	  if (r->chimera == 0)
+	    r->st.nmatches = 0;
+	}
+	for (curRes = 0; curRes < res.nb; curRes++) {
+	  result_p_t r = res.e.result[curRes];
+	  if (r->chimera == 0)
+	    r->st.nmatches = 0;
+	}
+      } else {
+	/* The chimera loses.  */
+	for (curRes = 0; curRes < rev_res.nb; curRes++) {
+	  result_p_t r = rev_res.e.result[curRes];
+	  if (r->chimera)
+	    r->st.nmatches = 0;
+	}
+	for (curRes = 0; curRes < res.nb; curRes++) {
+	  result_p_t r = res.e.result[curRes];
+	  if (r->chimera)
+	    r->st.nmatches = 0;
+	}
+      }
+    }
     /* Keep only the best matches, according to filterPct.  */
     if (options.filterPct > 0) {
       unsigned int max_nmatches = 0;
@@ -386,6 +473,8 @@ print_res(result_p_t res, int rev, seq_p_t seq1, seq_p_t seq2)
   unsigned int i;
   if (res->st.nmatches >= options.minScore_cutoff) {
     printf("\n%s%s\n", seq1->header, seq2->header);
+    if (res->chimera)
+      printf("Chimera\n\n");
     if (rev)
       printf("(complement)\n\n");
     switch (options.ali_flag) {
@@ -400,6 +489,7 @@ print_res(result_p_t res, int rev, seq_p_t seq1, seq_p_t seq2)
       print_align_lat(seq1->seq, seq2->seq, res);
       break;
     case 4:
+    case 5:
       print_exons(&res->eCol, res->direction);
       print_polyA_info(seq1, seq2, &res->eCol, &res->st);
       print_align_lat(seq1->seq, seq2->seq, res);
@@ -441,7 +531,7 @@ print_polyA_info(seq_p_t s1, seq_p_t s2, collec_p_t eCol,
 	   pos, i, s1->seq + e->to1, e->to1 + 1 + options.dnaOffset);
     if (e->to1 < scanLen)
       scanLen = e->to1;
-    strncpy(buf, (char *) s1->seq + e->to1 - scanLen, scanLen);
+    strncpy(buf, (char *) s1->seq + e->to1 - scanLen, (size_t) scanLen);
     buf[scanLen] = 0;
     pSig = strstr(buf, "AATAAA");
     if (pSig == NULL)
@@ -470,7 +560,7 @@ print_polyA_info(seq_p_t s1, seq_p_t s2, collec_p_t eCol,
 	   pos, s2->seq + (e->from2 - 1 - pos), e->from2 - 1,
 	   pos, i, s1->seq + (e->from1 - 1 - i),
 	   e->from1 - 1 + options.dnaOffset);
-    strncpy(buf, (char *) s1->seq + e->from1 - 1, 50);
+    strncpy(buf, (char *) s1->seq + e->from1 - 1, 50UL);
     buf[50] = 0;
     pSig = strstr(buf, "TTTATT");
     if (pSig == NULL)
@@ -489,13 +579,15 @@ print_align_lat(uchar *seq1, uchar *seq2, result_p_t r)
 
   if (r->sList == NULL)
     return;
+  if (r->chimera && options.ali_flag < 5)
+    return;
   aligns = r->sList;
   while (aligns != NULL) {
     head = aligns;
     aligns = aligns->next_script;
     S = (int *) xmalloc((2 * head->len2 + 1 + 1) * sizeof(int));
     S++;
-    S2A(head->script, S, 0);
+    S2A(head->script, S);
     Free_script(head->script);
     IDISPLAY(seq1 + head->offset1 - 1 - 1, seq2 + head->offset2 - 1 - 1,
 	     head->len1, head->len2, S,
@@ -550,7 +642,7 @@ shuffle_line(read_buf_p_t b, size_t *cur)
     /* Adjust the input buffer.  */
     if (*cur < b->ic) {
       memmove(b->in, b->in + *cur, (b->ic - *cur) * sizeof(char));
-      b->ic -= *cur;
+      b->ic -= (unsigned int) *cur;
     } else
       b->ic = 0;
     *cur = 0;
@@ -571,12 +663,13 @@ read_line_buf(read_buf_p_t b, int fd)
   if ((s = shuffle_line(b, &cur)) != NULL)
     return s;
   do {
-    if ((rc = read(fd, b->in + b->ic, BUF_SIZE - b->ic - 1)) == -1) {
+    if ((rc = read(fd, b->in + b->ic,
+		   (size_t) (BUF_SIZE - b->ic - 1))) == -1) {
       if (errno != EINTR)
 	fatal("Could not read from %d: %s(%d)\n",
 	      fd, strerror(errno), errno);
     } else
-      b->ic += rc;
+      b->ic += (unsigned int) rc;
     s = shuffle_line(b, &cur);
     if (s == NULL && rc == 0) {
       /* Got to the EOF...  */
@@ -619,7 +712,7 @@ init_seq(const char *fName, seq_p_t sp)
 static int
 get_next_seq(seq_p_t sp, unsigned int offset, int warnMultiSeq)
 {
-  const int lenStr = 24;
+  const unsigned int lenStr = 24;
   unsigned int headerLen;
   char *buf = sp->rb.line;
   int res;
@@ -645,11 +738,11 @@ get_next_seq(seq_p_t sp, unsigned int offset, int warnMultiSeq)
       sp->seq = (unsigned char *)
 	xrealloc(sp->seq, sp->max * sizeof(unsigned char));
     }
-    while ((c = *buf++) != 0) {
+    while ((c = (unsigned char) *buf++) != 0) {
       if (isupper(c)) {
 	sp->seq[sp->len++] = c;
       } else if (islower(c)) {
-	sp->seq[sp->len++] = toupper(c);
+	sp->seq[sp->len++] = (unsigned char) toupper(c);
       }
     }
     buf = read_line_buf(&sp->rb, sp->fd);
@@ -676,8 +769,8 @@ get_next_seq(seq_p_t sp, unsigned int offset, int warnMultiSeq)
   buf = sp->header + headerLen - 1;
   while (iscntrl(*buf) || isspace(*buf))
     buf -= 1;
-  res = snprintf(buf + 1, lenStr, "; LEN=%u\n", sp->len + offset);
-  if (res < 0 || res >= lenStr)
+  res = snprintf(buf + 1, (size_t) lenStr, "; LEN=%u\n", sp->len + offset);
+  if (res < 0 || res >= (int) lenStr)
     fatal("Sequence too long: %u\n", sp->len);
   return 0;
 }
